/// <reference types="node" />
/// <reference types="node" />
import { Decoder } from '../RESP/decoder';
import { CommandArguments, Flags, RespVersions } from '../RESP/types';
import { ChannelListeners, PubSubListener, PubSubType, PubSubTypeListeners } from './pub-sub';
import { EventEmitter } from 'stream';
export interface QueueCommandOptions {
    asap?: boolean;
    chainId?: symbol;
    signal?: AbortSignal;
    flags?: Flags;
}
export interface CommandWaitingToBeSent extends CommandWaitingForReply {
    args: CommandArguments;
    chainId?: symbol;
    removeAbortListener?(): void;
}
interface CommandWaitingForReply {
    resolve(reply?: unknown): void;
    reject(err: unknown): void;
    channelsCounter?: number;
    flags?: Flags;
}
export type OnShardedChannelMoved = (channel: string, listeners: ChannelListeners) => void;
export default class RedisCommandsQueue {
    #private;
    private readonly _maxLength;
    private readonly _waitingToBeSent;
    private readonly _waitingForReply;
    private readonly _onShardedChannelMoved;
    private readonly _pubSub;
    get isPubSubActive(): boolean;
    private _chainInExecution;
    decoder: Decoder;
    constructor(respVersion: RespVersions | null | undefined, maxLength: number | null | undefined, onShardedChannelMoved: EventEmitter['emit']);
    private _initiateDecoder;
    private _onReply;
    private _onErrorReply;
    private _onPush;
    private _getFlags;
    private _initiateResp3Decoder;
    private _initiateResp2Decoder;
    addCommand<T>(args: CommandArguments, options?: QueueCommandOptions): Promise<T>;
    subscribe<T extends boolean>(type: PubSubType, channels: string | Array<string>, listener: PubSubListener<T>, returnBuffers?: T): Promise<void> | undefined;
    unsubscribe<T extends boolean>(type: PubSubType, channels?: string | Array<string>, listener?: PubSubListener<T>, returnBuffers?: T): Promise<void> | undefined;
    resubscribe(): Promise<any> | undefined;
    extendPubSubChannelListeners(type: PubSubType, channel: string, listeners: ChannelListeners): Promise<void> | undefined;
    extendPubSubListeners(type: PubSubType, listeners: PubSubTypeListeners): Promise<void> | undefined;
    getPubSubListeners(type: PubSubType): PubSubTypeListeners;
    private _pushPubSubCommand;
    getCommandToSend(): CommandArguments | undefined;
    flushWaitingForReply(err: Error): void;
    flushAll(err: Error): void;
}
export {};
