/// <reference types="node" />
import { RedisClusterOptions } from '.';
import { RedisClientType } from '../client';
import { EventEmitter } from 'stream';
import { ChannelListeners } from '../client/pub-sub';
import { RedisArgument, RedisFunctions, RedisModules, RedisScripts, RespVersions } from '../RESP/types';
interface NodeAddress {
    host: string;
    port: number;
}
export type NodeAddressMap = {
    [address: string]: NodeAddress;
} | ((address: string) => NodeAddress | undefined);
type ValueOrPromise<T> = T | Promise<T>;
type ClientOrPromise<M extends RedisModules, F extends RedisFunctions, S extends RedisScripts, RESP extends RespVersions = 2> = ValueOrPromise<RedisClientType<M, F, S, RESP>>;
export interface Node<M extends RedisModules, F extends RedisFunctions, S extends RedisScripts, RESP extends RespVersions> {
    address: string;
    client?: ClientOrPromise<M, F, S, RESP>;
}
export interface ShardNode<M extends RedisModules, F extends RedisFunctions, S extends RedisScripts, RESP extends RespVersions> extends Node<M, F, S, RESP> {
    host: string;
    port: number;
    readonly: boolean;
}
export interface MasterNode<M extends RedisModules, F extends RedisFunctions, S extends RedisScripts, RESP extends RespVersions> extends ShardNode<M, F, S, RESP> {
    pubSubClient?: ClientOrPromise<M, F, S, RESP>;
}
export interface Shard<M extends RedisModules, F extends RedisFunctions, S extends RedisScripts, RESP extends RespVersions> {
    master: MasterNode<M, F, S, RESP>;
    replicas?: Array<ShardNode<M, F, S, RESP>>;
    nodesIterator?: IterableIterator<ShardNode<M, F, S, RESP>>;
}
export type PubSubNode<M extends RedisModules, F extends RedisFunctions, S extends RedisScripts, RESP extends RespVersions> = Required<Node<M, F, S, RESP>>;
export type OnShardedChannelMovedError = (err: unknown, channel: string, listeners?: ChannelListeners) => void;
export default class RedisClusterSlots<M extends RedisModules, F extends RedisFunctions, S extends RedisScripts, RESP extends RespVersions> {
    #private;
    private static _SLOTS;
    private readonly _options;
    private readonly _clientFactory;
    private readonly _emit;
    slots: Shard<M, F, S, RESP>[];
    shards: Shard<M, F, S, RESP>[];
    masters: ShardNode<M, F, S, RESP>[];
    replicas: ShardNode<M, F, S, RESP>[];
    readonly nodeByAddress: Map<string, ShardNode<M, F, S, RESP> | MasterNode<M, F, S, RESP>>;
    pubSubNode?: PubSubNode<M, F, S, RESP>;
    private _isOpen;
    get isOpen(): boolean;
    constructor(options: RedisClusterOptions<M, F, S, RESP>, emit: EventEmitter['emit']);
    connect(): Promise<void>;
    private _discoverWithRootNodes;
    private _resetSlots;
    private _discover;
    private _getShards;
    private _getNodeAddress;
    private _clientOptionsDefaults;
    private _initiateSlotNode;
    private _createClient;
    private _createNodeClient;
    nodeClient(node: ShardNode<M, F, S, RESP>): ClientOrPromise<M, F, S, RESP>;
    rediscover(startWith: RedisClientType<M, F, S, RESP>): Promise<void>;
    private _rediscover;
    quit(): Promise<void>;
    disconnect(): Promise<void>;
    private _destroy;
    private _execOnNodeClient;
    getClient(firstKey: RedisArgument | undefined, isReadonly: boolean | undefined): ClientOrPromise<M, F, S, RESP>;
    private _iterateAllNodes;
    _randomNodeIterator?: IterableIterator<ShardNode<M, F, S, RESP>>;
    getRandomNode(): ShardNode<M, F, S, RESP>;
    private _slotNodesIterator;
    getSlotRandomNode(slotNumber: number): ShardNode<M, F, S, RESP>;
    getMasterByAddress(address: string): ClientOrPromise<M, F, S, RESP> | undefined;
    getPubSubClient(): ClientOrPromise<M, F, S, RESP>;
    private _initiatePubSubClient;
    executeUnsubscribeCommand(unsubscribe: (client: RedisClientType<M, F, S, RESP>) => Promise<void>): Promise<void>;
    getShardedPubSubClient(channel: string): ClientOrPromise<M, F, S, RESP>;
    executeShardedUnsubscribeCommand(channel: string, unsubscribe: (client: RedisClientType<M, F, S, RESP>) => Promise<void>): Promise<void>;
}
export {};
