"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _RedisClusterSlots_instances, _RedisClusterSlots_runningRediscoverPromise, _RedisClusterSlots_initiateShardedPubSubClient;
Object.defineProperty(exports, "__esModule", { value: true });
const errors_1 = require("../errors");
const client_1 = require("../client");
const util_1 = require("util");
const pub_sub_1 = require("../client/pub-sub");
// TODO: ?!
// We need to use 'require', because it's not possible with Typescript to import
// function that are exported as 'module.exports = function`, without esModuleInterop
// set to true.
const calculateSlot = require('cluster-key-slot');
class RedisClusterSlots {
    get isOpen() {
        return this._isOpen;
    }
    constructor(options, emit) {
        _RedisClusterSlots_instances.add(this);
        Object.defineProperty(this, "_options", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "_clientFactory", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "_emit", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "slots", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Array(RedisClusterSlots._SLOTS)
        });
        Object.defineProperty(this, "shards", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Array()
        });
        Object.defineProperty(this, "masters", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Array()
        });
        Object.defineProperty(this, "replicas", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Array()
        });
        Object.defineProperty(this, "nodeByAddress", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Map()
        });
        Object.defineProperty(this, "pubSubNode", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "_isOpen", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: false
        });
        _RedisClusterSlots_runningRediscoverPromise.set(this, void 0);
        Object.defineProperty(this, "_randomNodeIterator", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this._options = options;
        this._clientFactory = client_1.default.factory(options);
        this._emit = emit;
    }
    async connect() {
        if (this._isOpen) {
            throw new Error('Cluster already open');
        }
        this._isOpen = true;
        try {
            await this._discoverWithRootNodes();
        }
        catch (err) {
            this._isOpen = false;
            throw err;
        }
    }
    async _discoverWithRootNodes() {
        let start = Math.floor(Math.random() * this._options.rootNodes.length);
        for (let i = start; i < this._options.rootNodes.length; i++) {
            if (await this._discover(this._options.rootNodes[i]))
                return;
        }
        for (let i = 0; i < start; i++) {
            if (await this._discover(this._options.rootNodes[i]))
                return;
        }
        throw new errors_1.RootNodesUnavailableError();
    }
    _resetSlots() {
        this.slots = new Array(RedisClusterSlots._SLOTS);
        this.shards = [];
        this.masters = [];
        this.replicas = [];
        this._randomNodeIterator = undefined;
    }
    async _discover(rootNode) {
        this._resetSlots();
        const addressesInUse = new Set();
        try {
            const shards = await this._getShards(rootNode), promises = [], eagerConnect = this._options.minimizeConnections !== true;
            for (const { from, to, master, replicas } of shards) {
                const shard = {
                    master: this._initiateSlotNode(master, false, eagerConnect, addressesInUse, promises)
                };
                if (this._options.useReplicas) {
                    shard.replicas = replicas.map(replica => this._initiateSlotNode(replica, true, eagerConnect, addressesInUse, promises));
                }
                this.shards.push(shard);
                for (let i = from; i <= to; i++) {
                    this.slots[i] = shard;
                }
            }
            if (this.pubSubNode && !addressesInUse.has(this.pubSubNode.address)) {
                if (util_1.types.isPromise(this.pubSubNode.client)) {
                    promises.push(this.pubSubNode.client.then(client => client.disconnect()));
                    this.pubSubNode = undefined;
                }
                else {
                    promises.push(this.pubSubNode.client.disconnect());
                    const channelsListeners = this.pubSubNode.client.getPubSubListeners(pub_sub_1.PubSubType.CHANNELS), patternsListeners = this.pubSubNode.client.getPubSubListeners(pub_sub_1.PubSubType.PATTERNS);
                    if (channelsListeners.size || patternsListeners.size) {
                        promises.push(this._initiatePubSubClient({
                            [pub_sub_1.PubSubType.CHANNELS]: channelsListeners,
                            [pub_sub_1.PubSubType.PATTERNS]: patternsListeners
                        }));
                    }
                }
            }
            for (const [address, node] of this.nodeByAddress.entries()) {
                if (addressesInUse.has(address))
                    continue;
                if (node.client) {
                    promises.push(this._execOnNodeClient(node.client, client => client.disconnect()));
                }
                const { pubSubClient } = node;
                if (pubSubClient) {
                    promises.push(this._execOnNodeClient(pubSubClient, client => client.disconnect()));
                }
                this.nodeByAddress.delete(address);
            }
            await Promise.all(promises);
            return true;
        }
        catch (err) {
            this._emit('error', err);
            return false;
        }
    }
    async _getShards(rootNode) {
        const options = this._clientOptionsDefaults(rootNode);
        options.socket ??= {};
        options.socket.reconnectStrategy = false;
        options.RESP = this._options.RESP;
        const client = client_1.default.factory(this._options)(options);
        client.on('error', err => this._emit('error', err));
        await client.connect();
        try {
            // switch to `CLUSTER SHARDS` when Redis 7.0 will be the minimum supported version
            return await client.clusterSlots();
        }
        finally {
            await client.disconnect();
        }
    }
    _getNodeAddress(address) {
        switch (typeof this._options.nodeAddressMap) {
            case 'object':
                return this._options.nodeAddressMap[address];
            case 'function':
                return this._options.nodeAddressMap(address);
        }
    }
    _clientOptionsDefaults(options) {
        if (!this._options.defaults)
            return options;
        let socket;
        if (this._options.defaults.socket) {
            socket = options?.socket ? {
                ...this._options.defaults.socket,
                ...options.socket
            } : this._options.defaults.socket;
        }
        else {
            socket = options?.socket;
        }
        return {
            ...this._options.defaults,
            ...options,
            socket
        };
    }
    _initiateSlotNode(slotAddress, readonly, eagerConnent, addressesInUse, promises) {
        const address = `${slotAddress.host}:${slotAddress.port}`;
        addressesInUse.add(address);
        let node = this.nodeByAddress.get(address);
        if (!node) {
            node = {
                ...slotAddress,
                address,
                readonly,
                client: undefined
            };
            if (eagerConnent) {
                promises.push(this._createNodeClient(node));
            }
            this.nodeByAddress.set(address, node);
        }
        (readonly ? this.replicas : this.masters).push(node);
        return node;
    }
    async _createClient(node, readonly = node.readonly) {
        const client = this._clientFactory(this._clientOptionsDefaults({
            socket: this._getNodeAddress(node.address) ?? {
                host: node.host,
                port: node.port
            },
            readonly,
            RESP: this._options.RESP
        }));
        client.on('error', err => this._emit('error', err));
        await client.connect();
        return client;
    }
    _createNodeClient(node) {
        const promise = this._createClient(node)
            .then(client => {
            node.client = client;
            return client;
        })
            .catch(err => {
            node.client = undefined;
            throw err;
        });
        node.client = promise;
        return promise;
    }
    nodeClient(node) {
        return node.client ?? this._createNodeClient(node);
    }
    async rediscover(startWith) {
        __classPrivateFieldSet(this, _RedisClusterSlots_runningRediscoverPromise, __classPrivateFieldGet(this, _RedisClusterSlots_runningRediscoverPromise, "f") ?? this._rediscover(startWith)
            .finally(() => __classPrivateFieldSet(this, _RedisClusterSlots_runningRediscoverPromise, undefined, "f")), "f");
        return __classPrivateFieldGet(this, _RedisClusterSlots_runningRediscoverPromise, "f");
    }
    async _rediscover(startWith) {
        if (await this._discover(startWith.options))
            return;
        return this._discoverWithRootNodes();
    }
    quit() {
        return this._destroy(client => client.quit());
    }
    disconnect() {
        return this._destroy(client => client.disconnect());
    }
    async _destroy(fn) {
        this._isOpen = false;
        const promises = [];
        for (const { master, replicas } of this.shards) {
            if (master.client) {
                promises.push(this._execOnNodeClient(master.client, fn));
            }
            if (master.pubSubClient) {
                promises.push(this._execOnNodeClient(master.pubSubClient, fn));
            }
            if (replicas) {
                for (const { client } of replicas) {
                    if (client) {
                        promises.push(this._execOnNodeClient(client, fn));
                    }
                }
            }
        }
        if (this.pubSubNode) {
            promises.push(this._execOnNodeClient(this.pubSubNode.client, fn));
            this.pubSubNode = undefined;
        }
        this._resetSlots();
        this.nodeByAddress.clear();
        await Promise.allSettled(promises);
    }
    _execOnNodeClient(client, fn) {
        return util_1.types.isPromise(client) ?
            client.then(fn) :
            fn(client);
    }
    getClient(firstKey, isReadonly) {
        if (!firstKey) {
            return this.nodeClient(this.getRandomNode());
        }
        const slotNumber = calculateSlot(firstKey);
        if (!isReadonly) {
            return this.nodeClient(this.slots[slotNumber].master);
        }
        return this.nodeClient(this.getSlotRandomNode(slotNumber));
    }
    *_iterateAllNodes() {
        let i = Math.floor(Math.random() * (this.masters.length + this.replicas.length));
        if (i < this.masters.length) {
            do {
                yield this.masters[i];
            } while (++i < this.masters.length);
            for (const replica of this.replicas) {
                yield replica;
            }
        }
        else {
            i -= this.masters.length;
            do {
                yield this.replicas[i];
            } while (++i < this.replicas.length);
        }
        while (true) {
            for (const master of this.masters) {
                yield master;
            }
            for (const replica of this.replicas) {
                yield replica;
            }
        }
    }
    getRandomNode() {
        this._randomNodeIterator ??= this._iterateAllNodes();
        return this._randomNodeIterator.next().value;
    }
    *_slotNodesIterator(slot) {
        let i = Math.floor(Math.random() * (1 + slot.replicas.length));
        if (i < slot.replicas.length) {
            do {
                yield slot.replicas[i];
            } while (++i < slot.replicas.length);
        }
        while (true) {
            yield slot.master;
            for (const replica of slot.replicas) {
                yield replica;
            }
        }
    }
    getSlotRandomNode(slotNumber) {
        const slot = this.slots[slotNumber];
        if (!slot.replicas?.length) {
            return slot.master;
        }
        slot.nodesIterator ??= this._slotNodesIterator(slot);
        return slot.nodesIterator.next().value;
    }
    getMasterByAddress(address) {
        const master = this.nodeByAddress.get(address);
        if (!master)
            return;
        return this.nodeClient(master);
    }
    getPubSubClient() {
        return this.pubSubNode ?
            this.pubSubNode.client :
            this._initiatePubSubClient();
    }
    async _initiatePubSubClient(toResubscribe) {
        const index = Math.floor(Math.random() * (this.masters.length + this.replicas.length)), node = index < this.masters.length ?
            this.masters[index] :
            this.replicas[index - this.masters.length];
        this.pubSubNode = {
            address: node.address,
            client: this._createClient(node, true)
                .then(async (client) => {
                if (toResubscribe) {
                    await Promise.all([
                        client.extendPubSubListeners(pub_sub_1.PubSubType.CHANNELS, toResubscribe[pub_sub_1.PubSubType.CHANNELS]),
                        client.extendPubSubListeners(pub_sub_1.PubSubType.PATTERNS, toResubscribe[pub_sub_1.PubSubType.PATTERNS])
                    ]);
                }
                this.pubSubNode.client = client;
                return client;
            })
                .catch(err => {
                this.pubSubNode = undefined;
                throw err;
            })
        };
        return this.pubSubNode.client;
    }
    async executeUnsubscribeCommand(unsubscribe) {
        const client = await this.getPubSubClient();
        await unsubscribe(client);
        if (!client.isPubSubActive) {
            await client.disconnect();
            this.pubSubNode = undefined;
        }
    }
    getShardedPubSubClient(channel) {
        const { master } = this.slots[calculateSlot(channel)];
        return master.pubSubClient ?? __classPrivateFieldGet(this, _RedisClusterSlots_instances, "m", _RedisClusterSlots_initiateShardedPubSubClient).call(this, master);
    }
    async executeShardedUnsubscribeCommand(channel, unsubscribe) {
        const { master } = this.slots[calculateSlot(channel)];
        if (!master.pubSubClient)
            return Promise.resolve();
        const client = await master.pubSubClient;
        await unsubscribe(client);
        if (!client.isPubSubActive) {
            await client.disconnect();
            master.pubSubClient = undefined;
        }
    }
}
_RedisClusterSlots_runningRediscoverPromise = new WeakMap(), _RedisClusterSlots_instances = new WeakSet(), _RedisClusterSlots_initiateShardedPubSubClient = function _RedisClusterSlots_initiateShardedPubSubClient(master) {
    const promise = this._createClient(master, true)
        .then(client => {
        client.on('server-sunsubscribe', async (channel, listeners) => {
            try {
                await this.rediscover(client);
                const redirectTo = await this.getShardedPubSubClient(channel);
                redirectTo.extendPubSubChannelListeners(pub_sub_1.PubSubType.SHARDED, channel, listeners);
            }
            catch (err) {
                this._emit('sharded-shannel-moved-error', err, channel, listeners);
            }
        });
        master.pubSubClient = client;
        return client;
    })
        .catch(err => {
        master.pubSubClient = undefined;
        throw err;
    });
    master.pubSubClient = promise;
    return promise;
};
Object.defineProperty(RedisClusterSlots, "_SLOTS", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: 16384
});
exports.default = RedisClusterSlots;
